<a id="initialize-app"></a>

### Initialize an app.

If you has ever developed a vue app, you have already known that in a web app which is based on vue, there will be at least one root Vue view model. In Vert, we call every single root view model a `App`.

This example shows how to create an app instance to initialize your web app.

```html
<!-- HTML Template. -->

<html>
  <head>
    <title>App Index</title>
  </head>
  
  <body>
    <div id="app-index"></div>
  </body>
</html>
```

```typescript
// Bootstrap a project by creating a new instance of "App"
// App entry file.

import { App } from '@vert/core'
import RootComponent from './root-component/index.vue'

const app = new App({
  element: '#app-index',
  name: 'app-index',
  RootComponent: RootComponent  // Each App needs a component to initialize view.
})

app.start()
```

There can also be multi apps in your project:

```html
<html>
  <head>
    <meta charset="utf-8">
    <title>Multi apps in one page</title>
  </head>

  <body>
    <div id="header-vm"></div>
    <div id="main-vm"></div>
  </body>
</html>
```

```typescript
import { App } from '@vert/core'
import { Store } from 'vuex'

import HeaderComponent from './header/index.vue'
import MainComponent from './main/index.vue'

const headerVM = new App({
  element: '#header-vm',
  RootComponent: HeaderComponent,
  store: new Store({
    state: { name: 'Header VM' },
    getters: {
      getName (state) { return state.name }
    }
  })
})

const mainVM = new App({
  element: '#main-vm',
  RootComponent: MainComponent,
  store: new Store({
    state: { name: 'Main VM' },
    getters: {
      getName (state) { return state.name }
    }
  })
})

headerVM.start()
mainVM.start()
```

That's all, your app has been initialized.
