<a id="type-safety-in-run-time"></a>

### 运行时的类型安全

当您的项目变得庞大并且业务越来越重要时，您需要关注您的项目在运行时的 `类型安全` 问题。

就算您使用 `TypeScript` 进行代码编写，仍然需要进行必要的数据类型检查：

```typescript
// Bad: 当服务器返回非 string[] 数据时，可能会引起您的业务出错.
async function fetchUserList () {
  const userList: string[] = await http.get('/api/v1/user-list')
  return userList
}

// Good: 对数据进行类型检查，当有不符合类型时返回正确类型的默认值并记录 / 抛出问题.
async function fetchUserList () {
  const userList: string[] = await http.get('/api/v1/user-list')
  if (Array.isArray(userList)) {
    return userList.map(item => {
        if (typeof item === 'string') {
          return item
        } else {
          Logger.error(`fetchUserList() 中返回了错误的类型成员.`)
          return '--'
        }
      })  
  } else {
    Logger.error('fetchUserList() 返回了非 string[] 类型数据.')
    return []
  }
}
```

建议您最起码对关键业务进行数据类型检查，以保证程序基础功能不会出现问题，并在出错时对用户进行友好提示。

您可能认为这样做会很麻烦，但它是非常值得且应该去做的事情。
